/***
Controll is the holder for the root level model objects and viewer objects.
Models are created in the course of running handlers and are stored here.
Viewers register themselves here when they are created.
When the object of interest changes, each viewer in the list is notified of the
change so that that it can:
  1) move the data in the current view back into the model where it is 'safe'
  2) load the data for the newly selected object.
  3) display the data to the user.
***/
 

/** a map of every object name to it's HarmonyObjectDefinition **/
var objectMap = new Object();

/** a map of every HarmonyObjectDefinition viewer **/
var viewerMap = new Array();

/** the tag name of the object currently being viewed/edited **/
var curObjectName = null;

/** a map of the expanded state of various widgets in the system **/
var expandMap = new Object();

/** a map of the link codes in the current object, mapped to their comments **/
var linkCodeCommentPile = new Object();

/***
returns the currently selected object type
***/
function getCurObject()
{
  if(curObjectName == null){
    return null;
  }
  return objectMap[curObjectName];
}


var doubleLock = 0;

/***
update the view.
***/
function updateEditArea(objectToEdit)
{
  try{
    stopEdit(false);
    var objectDef = objectMap[objectToEdit];
    curObjectName = objectToEdit;

    label = document.getElementById("current-object-repeat");
    if(objectToEdit != null){
      label.value=objectToEdit;
      if(objectDef.needsDataLoad()){
        //--- get the object if I need it
        loadObjectDefinition();
      }
      else{
        //---- Update the view
        // ( Don't do this if we have to get the object because we have to wait for the 
        //   data to be loaded.  loadObjectDefinition() will ensure that the view is updated.
        updateEditAreaView();
      }
    }
    else{
      label.value="No Object Selected";
    }
    updatePalette();
  }
  catch(ex){
    dumpln(ex);
  }
}

/***
Sets the correct palette for the current tab
***/
function updatePalette()
{
  if(curObjectName != null){
    var mainTabBox = document.getElementById("main-tab-box");
    var mainTabBoxHide = document.getElementById("main-tab-box-hide");
    var newIndex = mainTabBox.selectedIndex + 1;
    document.getElementById("palette-deck").setAttribute("selectedIndex",newIndex);

    // put back for disappearing tabbox
    mainTabBoxHide.setAttribute("selectedIndex",1);
  }
  else{
    document.getElementById("palette-deck").setAttribute("selectedIndex",0);
    var mainTabBoxHide = document.getElementById("main-tab-box-hide");
    // put back for disappearing tabbox
    mainTabBoxHide.setAttribute("selectedIndex",0);
  }
}

/***
update the view with the newly loaded objects
***/
function updateEditAreaView()
{
  var objectDef = objectMap[curObjectName];

  //--- each call to update view will save widget to model and then move model to widgets
  var uvctr = -1;
  for(uvctr=0;uvctr<viewerMap.length;uvctr++){
    cur = viewerMap[uvctr];
    cur.updateView(objectDef);
  }

}


/***
This convenience method tells whatever the current object is that it has changed.
***/
function declareCurObjectHasChanged()
{
  if(null != getCurObject()){
    getCurObject().updated(true);
  }
}

/***
Often if the object palette it updated, the currently selected item is unselected.
This re-selects the item in the UI.


FOR REASONS UNKNOWN THIS METHOD DOES NOT WORK
***/  
function reSelectCurObject()
{
/*****
  try{
    dumpln("COMPARE TO:"+curObjectName);

    var localObjectPane = document.getElementById("power-object-pane");
    var indexCtr = 0;

    if(localObjectPane.childNodes != null){
      for(indexCtr=0; indexCtr<localObjectPane.childNodes.length ;indexCtr++){
        var curNode = localObjectPane.childNodes.item(indexCtr).firstChild;

        dumpln("::"+curNode.tagName);
        dumpln(":-"+curNode.getAttribute("label"));
        
        if(curNode.getAttribute("label") == curObjectName){
          dumpln("::"+curNode.getAttribute("label"));
//          localObjectPane.selectItem(curNode.parentNode);
          localObjectPane.selectItem(localObjectPane.childNodes.item(0));
          localObjectPane.selectItem(localObjectPane.childNodes.item(indexCtr+1));

          
          indexCtr = localObjectPane.childNodes.length;
          dumpln("--found it--");
        }
      }
    }
  }
  catch(e){dumpln(e);}
*****/  
}

