/***
This file holds all of the drag and drop handlers for the application.
These handlers may be used in a variety of widgets
***/

//================================================== Existing Question Drag

/***
Called when moving an existing question
***/
var moveQuestionObserver={
  onDragStart: function (evt,transferData,action){
    try{
      //--- the owning documents id for this tag is the data sent to the handler
      var questionId = alterId(evt.target.id,null);

      //--- remove this question from the given form id
      curObject = getCurObject();
      curObject.removeQuestionFromForm(questionId,evt.formId);

      formEditor = document.getElementById("form:"+evt.formId);
      formEditor = formEditor.parentNode;
      if(formEditor==null){
        dumpln("no form editor found");
      }

      formEditor.updateView(curObject);
      txt = "fq:"+questionId;

      if(txt==null){
        dumpln("You are trying to drag from something that is not properly configured.  Drag Aborted.");
        return;
      }

      transferData.data=new TransferData();
      transferData.data.addDataForFlavour("text/form-question",txt);  //puts the handler in with it's flavor info
    }
    catch(e){
      dumpln("EXC:"+e);
      getStackTrace();
    }
  }
};

/***
Used when dropping a moved question back into the list
***/
var questionListObserver={
  getSupportedFlavours : function () {
    var flavours = new FlavourSet();
    flavours.appendFlavour("text/form-question");
    return flavours;
  },
  onDragOver: function (evt,flavour,session){},
  onDrop: function (evt,dropdata,session){

    // grocery list:
    // form id being dropped on
    // question id of moving question

    try {
      //--- first get the nodes in motion

      var curBlock = evt.target;
      while (curBlock.tagName != "form-editor") {  // find the id for this power box entry, which is cached in the local groupbox.
        curBlock = curBlock.parentNode;
      }
      
      curObject = getCurObject();
      //--- pass in de-munged question id and form id and the index to put the question in at
      var cqId = alterId(dropdata.data,null);  // moving question id
      var cfId = alterId(curBlock.id,null);    // target form id
      var ctId = alterId(evt.target.id,null);  // target question location id
      curObject.addQuestionToFormAtIndex(cqId, cfId, ctId );
      curObject.updated(true);

      //--- Tell the UI to redraw itself
      document.getElementById("def-form-editor").updateView(curObject);

    } 
    catch (exc) {
      dumpln("EXCEPTION: "+exc);
      getStackTrace();
    }
  }
};

//================================================== New Question Drag

/***
Called when making a new question by pulling one off the palette
***/
var newQuestionObserver={
  onDragStart: function (evt,transferData,action){
    try {
      tree = evt.target.parentNode;
      //questionType = tree.view.getCellValue(tree.currentIndex,0);

      treeCell = tree.view.getItemAtIndex(tree.currentIndex).firstChild.firstChild;
      questionType = treeCell.getAttribute("label");

      model = objectMap[curObjectName];
      questionId = model.addNewQuestion(questionType); // the evt.target.id is the type to be used.  question id is for later reference of the new question

      if(questionId==null){
        dumpln("You are trying to drag from something that is not properly configured.  Drag Aborted.");
        return;
      }

      questionId = "q:"+questionId;

      transferData.data=new TransferData();
      transferData.data.addDataForFlavour("text/form-question",questionId);  //puts the handler in with it's flavor info
    } catch (e) {
      dumpln(e);
      getStackTrace();
    }
  }
};

//============================================= Drag forms

/***
Called when dragging a form
***/
this.moveFormObserver={
  onDragStart: function (evt,transferData,action){
  	var txt = alterId( evt.target.id, null);  // the owning documents id for this tag is the data sent to the handler
    if(txt==null){
      dumpln("You are trying to drag from something that is not properly configured.  Drag Aborted.");
      return;
    }
	  transferData.data=new TransferData();
	  transferData.data.addDataForFlavour("text/harmony-form",txt);  //puts the handler in with it's flavor info
  }
};

/***
Called when dropping a form
***/
this.formListObserver={
  getSupportedFlavours : function ()
  {
	  var flavours = new FlavourSet();
	  flavours.appendFlavour("text/harmony-form");
	  return flavours;
  },
  onDragOver: function (evt,flavour,session){},
  onDrop: function (evt,dropData,session)
  {

     curObject = objectMap[curObjectName];
     formList = curObject.getFormList();

     newFormList = new Array();

     movedFormId      = dropData.data;  // form being moved.

     dropTargetFormId = alterId( evt.target.id,null); // form being dropped on to
     refNode = null;                                  // the dom node to use for insertBefore()

     holdNode = null;

     insertIndex = -1; // where to put the holding node
	   order = null;  // tells if the form came from before or after the current one


     for(ctr=0;ctr<formList.length;ctr++){
       curForm = formList[ctr];
       curFormId = getChildValue(curForm,"anonid");
     }


	   for(ctr=0;ctr<formList.length;ctr++){
	     curForm = formList[ctr];
       curFormId = getChildValue(curForm,"id");

       if( (curFormId != movedFormId) &&
           (curFormId != dropTargetFormId)){
         //--- nothing happening, just copy it.
         newFormList[newFormList.length] = curForm;
       }

	     if(curFormId == movedFormId){
         //--- we found the moving node before the drop node.  Hold the moving node, out of the list.
         holdNode = curForm;
	     }

	     if(curFormId == dropTargetFormId ){
         if(holdNode != null){
           //--- we found the moving node before the drop target node.
           //    put the moved node aftr the dropped node.
           newFormList[newFormList.length] = curForm;
           insertIndex = newFormList.length;
           newFormList[newFormList.length] = "TEMP";
         }
         else{
           //--- we found the drop target node after the moving node.
           //    put the moved node before the dropped node.
           insertIndex = newFormList.length;
           newFormList[newFormList.length] = "TEMP";
           newFormList[newFormList.length] = curForm;
         }
	     }
	   }
     newFormList[insertIndex] = holdNode;  //--- put the form in its new location.

     curObject.setFormList(newFormList);

     damnList = curObject.getFormList();
     for(ctr=0;ctr<damnList.length;ctr++){
       curForm = damnList[ctr];
       curFormId = getChildValue(curForm,"id");
    }



     //--- Tell the UI to redraw itself
    document.getElementById("def-form-editor").updateView(curObject);
 }
};


//============================================= Drag Question Names


/***
??? Is this still used?
***/
var moveQuestionNameObserver={
  onDragStart: function (evt,transferData,action){
  	var txt = evt.target.label;  
    if(txt==null){
      dumpln("You are trying to drag from something that is not properly configured.  Drag Aborted.");
      return;
    }
	  transferData.data=new TransferData();
	  transferData.data.addDataForFlavour("text/unicode",txt);  //puts the handler in with it's flavor info
  }
};


//===============================================


/***
This is called when dragging a link off of the linkable object palette, but doesn't 
really do anything but put some junk in the transfer data.
***/
var moveLinkObjectObserver={
  onDragStart: function (evt,transferData,action){
    try{

      //dumpProps(evt);

      txt = "ASDF";
      transferData.data=new TransferData();
      transferData.data.addDataForFlavour("text/form-link",txt);  //puts the handler in with it's flavor info
    }
    catch(e){
      dumpln("EXC:"+e);
      getStackTrace();
    }
  }
};

/***
When a drop onto the link pane happens thia method interrogate the link page to 
see what is selected and adds a link to that type of object.  A dialog appears here
as well.
***/
var dropLinkObjectObserver={
  getSupportedFlavours : function () 
  {
    var flavours = new FlavourSet();
    flavours.appendFlavour("text/form-link");
    return flavours;
  },
  onDragOver: function (evt,flavour,session){},
  onDrop: function (event,dropdata,session)
  {
    try{
      var theTree = document.getElementById("link-definition-tree");
      var theColumnClicked = new Object();
      var theRowClicked = new Object();
      var theChildElement = new Object();

      theTree.treeBoxObject.getCellAt(event.pageX, event.pageY, theRowClicked, theColumnClicked, theChildElement);
      var targetLinkCode = null;

      // fifth column has the link code in label
      dumpln(">>"+theRowClicked.value);
      if(theRowClicked.value != -1){
        var columnList = getChildList(theTree.view.getItemAtIndex(theRowClicked.value).firstChild);
        targetLinkCode = columnList[5].getAttribute("label");  
      }

      //--- find the object type that was dragged.
      var theList = document.getElementById("linkable-object-palette");
      var draggedObjectName = theList.selectedItem.getAttribute("label")
      var returnData = new Object();

      window.openDialog("chrome://agilmente/content/NewLink.xul",
                        "new-link-dialog",
                        "chrome, modal, centerscreen",
                        curObjectName,       // near side of the link
                        draggedObjectName,   // far side of the link
                        targetLinkCode,      // link code dropped on
                        linkCodeCommentPile, // map of link codes to comments
                        returnData           // where the method puts results
                        );

      //===== bail if they canceled
      if(returnData.cancel=="true"){
        return;
      }
      //dumpProps(returnData);  // defined in control
      //      var curLinkDefNode = getDANode(localLinkList[0]);

      //====== prep the data
      var localLinkList = getCurObject().getLinkKeyList();
      var linkComment   = null;
      var linkCode      = returnData.resultSelectedLinkCode;

      if(linkCode == null || linkCode == ""){
        linkCode    = returnData.resultNewLinkCode;
        linkComment = returnData.resultNewLinkCodeComment;
      }
      else{
        linkComment = linkCodeCommentPile[returnData.resultSelectedLinkCode];
      }

      //====== Create a new power link tree
      var newPowerLink = createTemplateDOM("will be ignored");
      var plWrapper = addPowerLink(newPowerLink, linkCode, linkComment);  // this is the tag we really want
      plWrapper.setAttribute("needAction","define");
      addPowerLinkSide(plWrapper,curObjectName);
      addPowerLinkSide(plWrapper,draggedObjectName);
      addEmptyPowerLinkStatus(plWrapper);

      if(!linkDefListContains(getCurObject().getLinkKeyList(),getFullLinkDefName(plWrapper))){
        localLinkList[localLinkList.length] = makeNodeDirectlyAccessible(plWrapper);
        var localLinkCodeMap = getCurObject().getLinkCodeMap();
        localLinkCodeMap[linkCode] = localLinkList[localLinkList.length-1];

        declareCurObjectHasChanged();
        updateEditAreaView();
      }
    }
    catch(ex){
      dumpln("drop link:"+ex);
    }
  }
};



