
include ('chrome://jslib/content/io/file.js');
include ('chrome://jslib/content/io/dirUtils.js');
include ('chrome://jslib/content/io/fileUtils.js');

/***
define a FastProps object and pass in the name of the file to be used for storage
Do NOT pass in a path, just a name.  if you pass in MyProps the 
file [mozUserHome]/MyProps.props will be created and used.
***/
function FastProps(fileName)
{
  var du=new DirUtils();
  var fu=new FileUtils();

  // initialize the member variables for this instance
  this.fileName = fu.append(du.getMozUserHomeDir(),fileName+".props");

  // names of the properties to manage
  this.propsList = new Array();
  // map of the data loaded fromthe disk.  (All objects are just maps in js)
  this.fileValues = new Object();
  
  // initialize the member function references
  this.addProperty = addProperty;
  this.getValue = getValue;
  this.readProps = readProps;
  this.writeProps = writeProps;
}
  
/***
Add a property that should be tracked.  This property must have some tag in the 
document that can give and receive values.
The property parameter will be used like this:
document.getElementById(propName).value = [value from file];
***/
function addProperty(propName)
{
  this.propsList[this.propsList.length] = propName;
}

/***
Returns the value of the property as it exists in the file, NOT how it is in the UI.
***/
function getValue(propName)
{
  return this.fileValues[propName];
}
  
/***
Tells the object to read the file and put all of the properties it finds into their
UI objects.
***/
function readProps()
{
  try{
    //--- file io
    var pFile = new File(this.fileName);
    pFile.open('r');
    var fileContents = pFile.read();
    pFile.close();

    //--- parse each line
    var lineList = fileContents.split('\n');
    for(ctr=0;ctr<lineList.length;ctr++){
      line = lineList[ctr];
      nvp = line.split("=");
      this.fileValues[nvp[0]] = nvp[1];
    }

    //--- put the parsed data into the requested fileds

    for(ctr=0; ctr<this.propsList.length;ctr++){
      var tag = document.getElementById(this.propsList[ctr]);
      if(tag != null){
        tag.value=this.fileValues[this.propsList[ctr]];
      }
    }

  }
  catch(ex){
    alert(ex);
  }
}

/***
Tells the object to take all the data out of the UI objects and write it to the file.
This will completely over write the file with the new values.
***/
function writeProps()
{
  try{

    var propString = "";

    for(ctr=0; ctr<this.propsList.length;ctr++){
      var tag = document.getElementById(this.propsList[ctr]);
      if(tag != null){
        propString+= this.propsList[ctr]+"="+tag.value+"\n";
      }
      else{
        propString+= this.propsList[ctr]+"=null\n";      
      }
    }

    var pFile = new File(this.fileName);
    if(!pFile.exists()){
      pFile.create();
    }
    pFile.open('w');
    pFile.write(propString);
    pFile.close();
  }
  catch(ex){
    alert(ex);
  }
}


