

/* Place JavaScript functions that are *not* event handlers in this file.
 * Use agilmente-handlers.js for JavaScript event handlers.
 */

var p = new XMLHttpRequest();
var dparse = new DOMParser();
var dserial = new XMLSerializer();


/***
Create the dom root for a harmony command list
***/
function createCommandListDOM()
{
  var root;
  try{
    root   = document.implementation.createDocument("", "harmony-command-list", null);
  }
  catch(ex){
    dumpln("createCommandListDOM "+ex);
    getStackTrace();
  }
  return root;
}




/***
Create the dom root for a single harmony command
***/
function createTemplateDOM(userDn)
{
  var root;
  try{
    root   = document.implementation.createDocument("", "harmony", null);

    header = root.createElementNS(null,"header");
    sender = root.createElementNS(null,"sender");
    user   = root.createElementNS(null,"user");
    id1    = root.createElementNS(null,"id");
    id2    = root.createElementNS(null,"id");
    person = root.createElementNS(null,"person");

    job    = root.createElementNS(null,"job");
    target = root.createElementNS(null,"target");
    nonwf  = root.createElementNS(null,"nonworkflow");
    action = root.createElementNS(null,"action");
    name   = root.createElementNS(null,"name");

    // header
    root.firstChild.appendChild(header);
    header.appendChild(sender);
    sender.appendChild(user);
    user.appendChild(id1);
    id1.appendChild(root.createTextNode(userDn));
    user.appendChild(person);
    person.appendChild(id2);
    id2.appendChild(root.createTextNode(userDn));

    // job
    root.firstChild.appendChild(job);
    job.appendChild(target);
    target.appendChild(nonwf);
    job.appendChild(action);
    action.appendChild(name);
  }
  catch(ex){
    dumpln("create template dom: "+ex);
    getStackTrace();
  }
  return root;
}

/*
Set the action into the given harmony call
*/
function setAction(root, action)
{
  try{
    nList = root.getElementsByTagName("action");
    nList = nList.item(0).getElementsByTagName("name");
    name = nList.item(0);
    name.appendChild(root.createTextNode(action));
    dumpln("  action request: "+action);
  }
  catch(ex){
    dumpln("setAction "+ex);
    getStackTrace();
  }
}


/***
gets a new organization node with the given org id
***/
function appendOrganization(root, orgId)
{
  try{
    nList = root.getElementsByTagName("nonworkflow");
    nwf = nList.item(0);
    orgEl = root.createElementNS(null,"organization");
    idEl = root.createElementNS(null,"id");

    nwf.appendChild(orgEl);
    orgEl.appendChild(idEl);
    idEl.appendChild(root.createTextNode(orgId));

  }
  catch(ex){
    dumpln("appendOrganization "+ex);
    getStackTrace();
  }
}

/***
Append a key to a command
***/
function appendKey(root, id, name, keyType, keyCode)
{
  try{
    nList = root.getElementsByTagName("nonworkflow");
    nwf = nList.item(0);

    var key = addTextTag(root,nwf,"key",null);
    if(id!=null){
      addTextTag(root,key,"id",id);
    }
    if(name!=null){
      addTextTag(root,key,"name",name);
    }
    if(keyType!=null){
      addTextTag(root,key,"key-type",keyType);
    }
    if(keyCode!=null){
      addTextTag(root,key,"key-code",keyCode);
    }
  }
  catch(ex){
    dumpln("appendKey "+ex);
    getStackTrace();
  }
}

/***
Append a key instance to a command
***/
function appendKeyInstance(root, keyInstId, keyId, serialNo)
{
  try{
    nList = root.getElementsByTagName("nonworkflow");
    nwf = nList.item(0);

    var keyInstTag = addTextTag(root,nwf,"key-instance",null);

    if(keyInstId!=null){
      addTextTag(root,keyInstTag,"id",keyInstId);
    }
    if(serialNo!=null){
      addTextTag(root,keyInstTag,"serial-number",serialNo);
    }
    if(keyId!=null){
      var keySideTag = addTextTag(root,keyInstTag,"key-side",null);
      var keyTag = addTextTag(root,keySideTag,"key",null);
      addTextTag(root,keyTag,"id",keyId);

    }
  }
  catch(ex){
    dumpln("appendKeyInstance "+ex);
    getStackTrace();
  }
}

/***
Append an access point to a command
***/
function appendAccessPoint(root, id, name, type, description)
{
  try{
    nList = root.getElementsByTagName("nonworkflow");
    nwf = nList.item(0);

    var accessPoint = addTextTag(root,nwf,"access-point",null);
    if(id!=null){
      addTextTag(root,accessPoint,"id",id);
    }
    if(name!=null){
      addTextTag(root,accessPoint,"name",name);
    }
    if(type!=null){
      addTextTag(root,accessPoint,"access-point-type",type);
    }
    if(description!=null){
      addTextTag(root,accessPoint,"description",description);
    }
  }
  catch(ex){
    dumpln("appendAccessPoint "+ex);
    getStackTrace();
  }
}

/***
adds a domain for logging in.
***/
function addDomain(root, user, pword, org)
{
  try{
    nList = root.getElementsByTagName("nonworkflow");
    nwf = nList.item(0);

    domainEl = root.createElementNS(null,"domain");
    nwf.appendChild(domainEl);

    idEl = root.createElementNS(null,"id");
    idEl.appendChild(root.createTextNode("dc=harmony"));
    domainEl.appendChild(idEl);
    
    
    userEl    = root.createElementNS(null,"user");
    unameEl   = root.createElementNS(null,"user-name");
    pwordEl   = root.createElementNS(null,"password");
    orgEl     = root.createElementNS(null,"organization");
    orgNameEl = root.createElementNS(null,"name");

    domainEl.appendChild(userEl);
    userEl.appendChild(unameEl);
    userEl.appendChild(pwordEl);
    userEl.appendChild(orgEl);

    unameEl.appendChild(root.createTextNode(user));
    pwordEl.appendChild(root.createTextNode(pword));
    orgEl.appendChild(orgNameEl);
    orgNameEl.appendChild(root.createTextNode(org));
  }
  catch(ex){
    dumpln("addDomain "+ex);

    getStackTrace();
  }
}




/***
gets a new organization node with the given org id
***/
function addComponentTag(root)
{
  try{
    nList = root.getElementsByTagName("nonworkflow");
    nwf = nList.item(0);
    orgEl = root.createElementNS(null,"component");

    nwf.appendChild(orgEl);
    orgEl.appendChild(idEl);
  }
  catch(ex){
    dumpln("addComponentTag "+ex);

    getStackTrace();
  }
}



/***
add filter
accepts the root to add the filter to and returns the filter element.
***/
function addFilterTag(root)
{
  try{
    nwf = root.getElementsByTagName("job").item(0);
    filterEl = root.createElementNS(null,"filter");

    nwf.appendChild(filterEl);
    return filterEl;
  }
  catch(ex){
    dumpln("addFilterTag "+ex);

    getStackTrace();
  }
  return null;
}
       
/***
add sort filter
accepts the root to add the filter to and returns the filter element.
***/
function addSortFilter(filterTag, column, direction, coltype)
{
  try{
    doc = filterTag.ownerDocument;
    sortFilter = doc.createElementNS(null,"sort-filter");
    filterTag.appendChild(sortFilter);

    col  = doc.createElementNS(null,"filter-column");
    col.appendChild(doc.createTextNode(column));
    dir  = doc.createElementNS(null,"filter-direction");
    dir.appendChild(doc.createTextNode(direction));
    type = doc.createElementNS(null,"filter-column-type");
    type.appendChild(doc.createTextNode(coltype));

    sortFilter.appendChild(col);
    sortFilter.appendChild(dir);
    sortFilter.appendChild(type);
  }
  catch(ex){
    dumpln("addSortFilter "+ex);

    getStackTrace();
  }
}

/***
Adds a form availability tag for listing
***/
function addFormAvailability(root, tagName, formId)
{
  try{
    nwf = root.getElementsByTagName("nonworkflow").item(0);
    wrapper = root.createElementNS(null,"form-availability");
    childWrapper = root.createElementNS(null,tagName+"-side");
    child = root.createElementNS(null,tagName);
    wrapper.appendChild(childWrapper);
    childWrapper.appendChild(child);

    if(formId != null){
      childWrapper = root.createElementNS(null,"form-side");
      child = root.createElementNS(null,"form");
      setChildValue(child,"id",formId);
      wrapper.appendChild(childWrapper);
      childWrapper.appendChild(child);
    }

    nwf.appendChild(wrapper);
  }
  catch(ex){
    dumpln("params :"+root+","+tagName+","+formId);
    dumpln("addFormAvailability: "+ex);
    getStackTrace();
  }
}

/***
Adds an empty power link and returns it so that it can be filled appropriately 
for its use
***/
function addPowerLink(root, linkCode, comment)
{
  try{
    var nwf = root.getElementsByTagName("nonworkflow").item(0);
    var wrapper = root.createElementNS(null,"power-link");

    if(linkCode != null){
      var linkCodeTag      = root.createElementNS(null,"link-code");
      linkCodeTag.appendChild(root.createTextNode( linkCode ));
      wrapper.appendChild(linkCodeTag);
    }

    if(comment != null){
      var commentTag      = root.createElementNS(null,"comment");
      commentTag.appendChild(root.createTextNode( comment ));
      wrapper.appendChild(commentTag);
    }

    nwf.appendChild(wrapper);
    return wrapper;
  }
  catch(ex){
    dumpln("addPowerLink "+ex);
    getStackTrace();
  }
  return null;
}

/***
Adds a side tag to an existing power link tag
***/
function addPowerLinkSide(powerLinkTag, sideTagName, sideId)
{
  try{
    var root = powerLinkTag.ownerDocument;
    var sideWrapper = root.createElementNS(null,sideTagName+"-side");
    var sideTag     = root.createElementNS(null,sideTagName);
    sideWrapper.appendChild(sideTag);
    
    if(sideId != null){
      var sideIdTag      = root.createElementNS(null,"id");
      sideIdTag.appendChild(root.createTextNode( sideId ));
      sideTag.appendChild(sideIdTag);
    }
    powerLinkTag.appendChild(sideWrapper);
  }
  catch(ex){
    dumpln("addPowerLinkSide "+ex);

    getStackTrace();
  }
}

/***
Adds a side tag to an existing power link tag
***/
function addPowerLinkStatus(powerLinkTag, statusMap, statusCodeName)
{
  try{
    var root = powerLinkTag.ownerDocument;
    var statusWrapper = root.createElementNS(null,"status");
    var statusTag     = root.createElementNS(null,statusCodeName);
    statusWrapper.appendChild(statusTag);
    powerLinkTag.appendChild(statusWrapper);
    
    for (var i in statusMap) {
      try{
        var statusEntryTag = root.createElementNS(null,i);
        statusEntryTag.appendChild(root.createTextNode( statusMap[i] ));
        statusTag.appendChild(statusEntryTag);
      }
      catch(e){ 
        dumpln("addPowerLinkStatus "+e);

        getStackTrace();
      }
    }
  }
  catch(ex){
    dumpln("addPowerLinkStatus "+ex);

    getStackTrace();
  }
}

/***
Adds a side tag to an existing power link tag
***/
function addEmptyPowerLinkStatus(powerLinkTag)
{
  try{
    var root = powerLinkTag.ownerDocument;
    var statusWrapper = root.createElementNS(null,"status");
    powerLinkTag.appendChild(statusWrapper);
  }
  catch(ex){
    dumpln("addEmptyPowerLinkStatus "+ex);

    getStackTrace();
  }
}



/***
Adds an empty custom object tag with the attribute set to the object type.
***/
function addCustomObject(root, objType)
{
  try{
    nwf = root.getElementsByTagName("nonworkflow").item(0);
    mainTag =  root.createElementNS(null,"custom-object");
    mainTag.setAttribute("type",objType);
    nwf.appendChild(mainTag);
  }
  catch(ex){
    dumpln("addCustomObject "+ex);
    getStackTrace();
  }
}


/***
Adds a a new custom object definition of the given name to the given document
***/
function addCustomObjectDefinition(root, newObjName)
{
  try{
    nwf = root.getElementsByTagName("nonworkflow").item(0);
    mainTag =  root.createElementNS(null,"custom-object");
    singular    = root.createElementNS(null,"pretty-name");
    singular.appendChild(root.createTextNode(newObjName));
    plural      = root.createElementNS(null,"pretty-name-plural");
    plural.appendChild(root.createTextNode(newObjName+"s"));
    defaultTag  = root.createElementNS(null,"default-name");
    defaultTag.appendChild(root.createTextNode("singular"));
    objectTag   = root.createElementNS(null,"type-name");
    objectTag.appendChild(root.createTextNode( makeTagName(newObjName) ));

    mainTag.appendChild(singular);
    mainTag.appendChild(plural);
    mainTag.appendChild(defaultTag);
    mainTag.appendChild(objectTag);

    nwf.appendChild(mainTag);
  }
  catch(ex){
    dumpln("addCustomObjectDefinition "+ex);

    getStackTrace();
  }
}

/***
make a tag name out of a string
***/
function makeTagName(inputName)
{
  inputName = inputName.toLowerCase();

  regex = /\W+/g;
  inputName = inputName.replace(regex,"-");
  return inputName;
}


/***
set the value of the nonworkflow tag to be whatever dom was sent in.
Note that I must use import node so that the roots don't get wonky.
***/
function setSubject(root, subject)
{
  try{
    nwf = root.getElementsByTagName("nonworkflow").item(0);
    //var newNode = root.importNode(subject,true);  // not currently supported by mozilla
    //nwf.appendChild(newNode);
    safeImportNode(nwf,subject);
  }
  catch(ex){

    dumpln("setSubject: "+ex);
    getStackTrace();
  }
}

/***
Unit test for safeImportNode
***/
function safeImportNodeTest()
{
  try{

  var doco1   = document.implementation.createDocument("", "harmony", null);
  var doco2   = document.implementation.createDocument("", "harmony", null);

  var header  = doco1.createElementNS(null,"header");
  var header2 = doco1.createElementNS(null,"header");
  var header3 = doco1.createElementNS(null,"header");

  var headerd2 = doco2.createElementNS(null,"header");


  header.appendChild(header2);
  header2.appendChild(header3);

  header.setAttribute("ATR1","ATRV1");
  header.setAttribute("ATR2","ATRV2");
  header.setAttribute("ATR3","ATRV3");
  header.setAttribute("ATR4","ATRV4");

  header2.setAttribute("ATR1","ATRV1");
  header2.setAttribute("ATR2","ATRV2");
  header2.setAttribute("ATR3","ATRV3");
  header2.setAttribute("ATR4","ATRV4");

  header3.setAttribute("ATR1","ATRV1");
  header3.setAttribute("ATR2","ATRV2");
  header3.setAttribute("ATR3","ATRV3");
  header3.setAttribute("ATR4","ATRV4");
  header3.appendChild(doco1.createTextNode( "Some Lovely Text with '!@#$%^ in it" ));

  safeImportNode(headerd2,header)
  /*
  dumpln("=== CopyResult");
  dumpln( prettyPrintDom(headerd2) );
  dumpln("=== EndCopyResult");
  */

  }
  catch(ex){
    dumpln("safeImportNodeTest: "+ex);
  }
}

/***
I'm irritated that I have to write this.  importNode doesn't work so I have to implement
my own cheese ball copy of it.
***/
function safeImportNode(parent, alien)
{
  var newNode = parent.ownerDocument.createElementNS(null,alien.tagName);

  var actr=0;
  if(alien.attributes != null){
    for(actr=0;actr<alien.attributes.length;actr++){
      var curAttr = alien.attributes.item(actr);
      newNode.setAttribute(curAttr.nodeName,curAttr.nodeValue);
    }
  }

  if(alien.childNodes!=null){
    for(actr=0;actr<alien.childNodes.length;actr++){
      var curNode = alien.childNodes.item(actr);
      if(curNode.nodeType == 1){  // Element Node
        safeImportNode(newNode,curNode);
      }
      if(curNode.nodeType == 3){  // Text Node
        newNode.appendChild(parent.ownerDocument.createTextNode(curNode.nodeValue));
      }
    }
  }

  parent.appendChild(newNode);
}


/***
Adds a harmony command to a command list
***/
function addToCommandList(commandList, command, wantResult, varName)
{
  try{
    var commandWrapper = commandList.createElementNS(null,"harmony-command");
    commandList.firstChild.appendChild(commandWrapper);
    safeImportNode(commandWrapper,command);
    if(wantResult!=null){
      var wantResultWrapper = command.ownerDocument.createElementNS(null,"want-result");
      wantResultWrapper.appendChild(command.ownerDocument.createTextNode(wantResult));
      commandWrapper.appendChild(wantResultWrapper);
    }
    if(varName!=null){
      var varNameWrapper = command.ownerDocument.createElementNS(null,"result-var");
      varNameWrapper.appendChild(command.ownerDocument.createTextNode(varName));
      commandWrapper.appendChild(varNameWrapper);
    }

  }
  catch(e){
    dumpln("addToCommandList: "+e);
  }

}



/****
A more ruggedized utility for adding a tag with a value for use with harmony dom
****/
function addTextTag(root, parentNode, tagName, tagValue)
{
  try{
    var localNode = root.createElementNS(null,tagName);
    if(tagValue!=null){
      localNode.appendChild(root.createTextNode(tagValue));
    }
    parentNode.appendChild(localNode);
    return localNode;
  }
  catch(ex){
    dumpln("addTextTag: "+ex);
    getStackTrace();
  }
  return null;
}


/***
Put in a time entry tag
***/
function addTimeEntry(root, userDn, start, end)
{
  try{
    nwf = root.getElementsByTagName("nonworkflow").item(0);
    mainTag =  root.createElementNS(null,"time-entry");

    userTag    =  root.createElementNS(null,"user");
    personTag  =  root.createElementNS(null,"person");
    idTag      =  root.createElementNS(null,"id");
    idTag.appendChild(root.createTextNode(userDn));
    id2Tag     =  root.createElementNS(null,"id");
    id2Tag.appendChild(root.createTextNode(userDn));
    beginTag   =  root.createElementNS(null,"begin-date");
    beginTag.appendChild(root.createTextNode(getSimpleDate(start)));
    endTag     =  root.createElementNS(null,"end-date");
    endTag.appendChild(root.createTextNode(getSimpleDate(end)));


    nwf.appendChild(mainTag);
    mainTag.appendChild(userTag);
    mainTag.appendChild(beginTag);
    mainTag.appendChild(endTag);
    userTag.appendChild(idTag);
    userTag.appendChild(personTag);
    personTag.appendChild(id2Tag);
  }
  catch(ex){
    dumpln("addTimeEntry "+ex);
    getStackTrace();
  }

}


function addCalendarDay(parentTag, targetDate)
{
  try{
    var root = parentTag.ownerDocument;
    mainTag =  root.createElementNS(null,"calendar-day");

    dateTag    =  root.createElementNS(null,"entry-date");
    dateTag.appendChild(root.createTextNode(getSimpleDate(targetDate)));

    hoursTag  =  root.createElementNS(null,"hours-worked");
    hoursTag.appendChild(root.createTextNode("0"));

    statusTag      =  root.createElementNS(null,"status");
    statusTag.appendChild(root.createTextNode("revoked"));

    mainTag.appendChild(dateTag);
    mainTag.appendChild(hoursTag);
    mainTag.appendChild(statusTag);

    parentTag.appendChild(mainTag);

    return mainTag;
  }
  catch(ex){
    dumpln("addCalendarDay "+ex);
    getStackTrace();
  }
}


/***
Return a harmony formatted date string from the javascript date object.  This version
ignores the hours
***/
function getSimpleDate(theDate)
{
  return (theDate.getMonth()+1)+"/"+theDate.getDate()+"/"+(1900+theDate.getYear())+" 00:00:00 PM, GMT"
}
