
var dumpln;
var dd;

// This may grow to be very large, but in situations where dumpln isn't available
// it provides an alternative.
var curLog;  


if (typeof document == "undefined") /* in xpcshell */
    dumpln = print;
else
    if (typeof dump == "function")
        dumpln = function (str) {
          if(curLog==null){
            curLog="";
          }
          dump (str + "\n"); 

          var LOG_SIZE = 300000;

          if(curLog.length>LOG_SIZE){
            curLog=curLog.slice(curLog.length-LOG_SIZE, curLog.length);
          }
          curLog += (str + "\n");
          dump(":::"+curLog);

        }
    else if (jsenv.HAS_RHINO)
        dumpln = function (str) {var out = java.lang.System.out;
                                 out.println(str); out.flush(); }
    else
        dumpln = function () {} /* no suitable function */

if (DEBUG)
    dd = dumpln;
else
    dd = function (){};

var jsenv = new Object();
jsenv.HAS_SECURITYMANAGER = ((typeof netscape == "object") &&
                             (typeof netscape.security == "object"));
jsenv.HAS_XPCOM = ((typeof Components == "function") &&
                   (typeof Components.classes == "function"));
jsenv.HAS_JAVA = (typeof java == "object");
jsenv.HAS_RHINO = (typeof defineClass == "function");
jsenv.HAS_DOCUMENT = (typeof document == "object");





/* Dumps an object in tree format, recurse specifiec the the number of objects
 * to recurse, compress is a boolean that can uncompress (true) the output
 * format, and level is the number of levels to intitialy indent (only useful
 * internally.)  A sample dumpObjectTree (o, 1) is shown below.
 *
 * + parent (object)
 * + users (object)
 * | + jsbot (object)
 * | + mrjs (object)
 * | + nakkezzzz (object)
 * | *
 * + bans (object)
 * | *
 * + topic (string) 'ircclient.js:59: nothing is not defined'
 * + getUsersLength (function) 9 lines
 * *
 */
function dumpObjectTree (o, recurse, compress, level)
{
    var s = "";
    var pfx = "";

    if (typeof recurse == "undefined")
        recurse = 0;
    if (typeof level == "undefined")
        level = 0;
    if (typeof compress == "undefined")
        compress = true;
    
    for (var i = 0; i < level; i++)
        pfx += (compress) ? "| " : "|  ";

    var tee = (compress) ? "+ " : "+- ";

    for (i in o)
    {
        
        var t = typeof o[i];
        switch (t)
        {
            case "function":
                var sfunc = String(o[i]).split("\n");
                if (sfunc[2] == "    [native code]")
                    sfunc = "[native code]";
                else
                    sfunc = sfunc.length + " lines";
                s += pfx + tee + i + " (function) " + sfunc + "\n";
                break;

            case "object":
                s += pfx + tee + i + " (object)\n";
                if (!compress)
                    s += pfx + "|\n";
                if ((i != "parent") && (recurse))
                    s += dumpObjectTree (o[i], recurse - 1,
                                         compress, level + 1);
                break;

            case "string":
                if (o[i].length > 200)
                    s += pfx + tee + i + " (" + t + ") " + 
                        o[i].length + " chars\n";
                else
                    s += pfx + tee + i + " (" + t + ") '" + o[i] + "'\n";
                break;

            default:
                s += pfx + tee + i + " (" + t + ") " + o[i] + "\n";
                
        }

        if (!compress)
            s += pfx + "|\n";

    }

    s += pfx + "*\n";
    
    return s;
    
}

/*
 * Clones an existing object (Only the enumerable properties
 * of course.) use as a function..
 * var c = Clone (obj);
 * or a constructor...
 * var c = new Clone (obj);
 */
function Clone (obj)
{
    robj = new Object();

    for (var p in obj)
        robj[p] = obj[p];

    return robj;
    
}

function renameProperty (obj, oldname, newname)
{

    if (oldname == newname)
        return;
    
    obj[newname] = obj[oldname];
    delete obj[oldname];
    
}

function newObject(contractID, iface)
{
    if (!jsenv.HAS_XPCOM)
        return null;

    var obj = Components.classes[contractID].createInstance();
    var rv;

    switch (typeof iface)
    {
        case "string":
            rv = obj.QueryInterface(Components.interfaces[iface]);
            break;

        case "object":
            rv = obj.QueryInterface[iface];
            break;

        default:
            rv = null;
            break;
    }

    return rv;
    
}

function keys (o)
{
    var rv = new Array();
    
    for (var p in o)
        rv.push(p);

    return rv;
    
}

function trim (s)
{
    if (!s)
        return "";
    s = s.replace (/^\s+/, "");
    return s.replace (/\s+$/, "");
    
}

function formatDateOffset (seconds, format)
{
    seconds = parseInt(seconds);
    var minutes = parseInt(seconds / 60);
    seconds = seconds % 60;
    var hours   = parseInt(minutes / 60);
    minutes = minutes % 60;
    var days    = parseInt(hours / 24);
    hours = hours % 24;

    if (!format)
    {
        var ary = new Array();
        if (days > 0)
            ary.push (days + " days");
        if (hours > 0)
            ary.push (hours + " hours");
        if (minutes > 0)
            ary.push (minutes + " minutes");
        if (seconds > 0)
            ary.push (seconds + " seconds");

        format = ary.join(", ");
    }
    else
    {
        format = format.replace ("%d", days);
        format = format.replace ("%h", hours);
        format = format.replace ("%m", minutes);
        format = format.replace ("%s", seconds);
    }
    
    return format;
}

function arraySpeak (ary, single, plural)
{
    var rv = "";
    
    switch (ary.length)
    {
        case 0:
            break;
            
        case 1:
            rv = ary[0];
            if (single)
                rv += " " + single;            
            break;

        case 2:
            rv = ary[0] + " and " + ary[1];
            if (plural)
                rv += " " + plural;
            break;

        default:
            for (var i = 0; i < ary.length - 1; ++i)
                rv += ary[i] + ", ";
            rv += "and " + ary[ary.length - 1];
            if (plural)
                rv += " " + plural;
            break;
    }

    return rv;
    
}


function arrayContains (ary, elem)
{
    return (arrayIndexOf (ary, elem) != -1);
}

function arrayIndexOf (ary, elem)
{
    for (var i in ary)
        if (ary[i] == elem)
            return i;

    return -1;
}
    
function arrayInsertAt (ary, i, o)
{

    ary.splice (i, 0, o);

}

function arrayRemoveAt (ary, i)
{

    ary.splice (i, 1);

}

function getRandomElement (ary)
{
    var i = parseInt (Math.random() * ary.length)
	if (i == ary.length) i = 0;

    return ary[i];

}

function roundTo (num, prec)
{

    return parseInt ( Math.round(num * Math.pow (10, prec))) /
        Math.pow (10, prec);   

}

function randomRange (min, max)
{

    if (typeof min == "undefined")
        min = 0;

    if (typeof max == "undefined")
        max = 1;

    var rv = (parseInt(Math.round((Math.random() * (max - min)) + min )));
    
    return rv;

}


function getInterfaces (cls)
{
    if (!jsenv.HAS_XPCOM)
        return null;

    var rv = new Object();
    var e;

    for (var i in Components.interfaces)
    {
        try
        {
            var ifc = Components.interfaces[i];
            cls.QueryInterface(ifc);
            rv[i] = ifc;
        }
        catch (e)
        {
            /* nada */
        }
    }

    return rv;
    
}



var prettyPrintSerializer = new XMLSerializer();

/***
Returns a nicely formmated string of a dom tree
***/
function prettyPrintDom(xmlDoc)
{
  return prettyPrintString(prettyPrintSerializer.serializeToString(xmlDoc.firstChild));
}

/***
Returns a nicely formatted version of an xml string
***/
function prettyPrintString(xmlString)
{
     txt=xmlString;
     list = txt.split('<');
     newtxt='';
     depth = 0;
     for(ctr=0;ctr<list.length;ctr++){
       if(ctr!=0){
         list[ctr]='<'+list[ctr]+'\n';
       }

       depth++;
       if(-1!=list[ctr].indexOf('</')){
         depth--;
       }

       for(pctr=0;pctr<depth;pctr++){
         newtxt=newtxt+' ';
       }
       newtxt=newtxt+list[ctr];

       if(-1!=list[ctr].indexOf('/>')){
         depth--;
       }
       if(-1!=list[ctr].indexOf('</')){
         depth--;
       }

     }

     return newtxt;
}

/***
Dumps out the tags and child tags of this element


USE THIS ONE INSTEAD OF PRETTY PRINT
***/
function dumpElement(element)
{
  if(element==null){
    dumpln("dumpElement:  passed in null element");
  }
  dumpElementRecur(element,0);
}

function dumpElementRecur(element,depth)
{
  var localCtr = 0;
  if(null != element.tagName){
    dumpln(depth+": "+element.tagName);
    for(localCtr=0;localCtr<element.childNodes.length;localCtr++){
      dumpElementRecur(element.childNodes.item(localCtr),depth+1);
    }
  }
  if(null != element.nodeValue && trim(element.nodeValue) != ""){
    dumpln(depth+"----: "+element.nodeValue);
  }
  if(9 == element.nodeType){
    dumpln(depth+": DOCUMENT PASSED INSTEAD OF ELEMENT");
  }
}

function firstChild(parentNode)
{
  var retList = new Array();

  try{
    if(parentNode==null){
      dumpln("firstChild: null parent node");
      return null;
    }

    for(gclctr=0;gclctr<parentNode.childNodes.length;gclctr++){
      if(parentNode.childNodes.item(gclctr).nodeType == 1){
        return parentNode.childNodes.item(gclctr);
      }
    }
  }
  catch(e){
    dumpln("firstChild: "+e);
    dumpln("input: "+parentNode);
    getStackTrace();
  }

  dumpln("firstChild: found no first child for "+parentNode.tagName);
  getStackTrace();

  return null;
}

/***
Does what DOM next sibling should do without the rampant stupidity of returning
empty text nodes.
***/
function nextSibling(parentNode)
{
  try{
    var retVal = parentNode.nextSibling;

    if(parentNode==null){
      dumpln("nextSibling: no start node given");
      return null;
    }

    while(retVal.nodeType != 1){  // not equal an element node
      retVal = retVal.nextSibling;
      if(retVal==null){
        dumpln("nextSibling: no next sibling found.");
        return null;
      }
    }

    return retVal;
  }
  catch(e){
    dumpln("nextSibling: "+e);
    dumpln("input: "+parentNode);
    getStackTrace();
  }
  return null;
}

/***
Debugging method to display what is available on an object.
***/
function dumpProps(obj) 
{
  dumpln("dumpProps: "+obj);
  if(obj==null){
    dumpln("dumpProps: object is null");
    getStackTrace();
    return;
  }

  var pctr = 0;
  for (var i in obj) {
    try{
      pctr++;
      dumpln(i);
      dumpln(" == [" + obj[i] +"]\n");; 
    }
    catch(e){ 
      getStackTrace();
      dumpln(e); 
    }
  }
  if(pctr==0){
    dumpln("dumpProps: object is not null, but it has no properties");
    getStackTrace();
  }
}

/***
Returns true if logging should be turned on.
***/
function isDebugMode()
{
  item = document.getElementById("menu_help_debug_log");
  if(item==null){
    return true;
  }
  if("true"==item.getAttribute("checked")){
    return true;
  }
  return false;
}


/***
skips some useless dom garbage and gets the value out of a node that contains text.
***/
function getDomValue(node)
{
  return node.firstChild.nodeValue;
}

/***
skips some useless dom garbage and gets the value out of a node that contains text.
***/
function getChildValue(node, tagName)
{
  var retVal = null;

  try{
    if(node==null){
      dumpln("getChildValue: bad node "+node);
      getStackTrace();
      return null;
    }
    nl = getChildList(node,tagName);
    if(nl==null){
      dumpln("getChildValue: bad child list "+nl);
      getStackTrace();
      return null;
    }
    if(nl[0]==null){
      return null;
    }
    if(nl[0].firstChild==null){
      dumpln("getChildValue: no first child on zeroth entry on "+nl[0].tagName);
      getStackTrace();
      return null;
    }
  
    retVal = nl[0].firstChild.nodeValue;
  }
  catch(e){
    dumpln("getChildValue: "+e);
    dumpln("input: "+node+","+tagName);
    getStackTrace();
  }

  if(retVal.length==null){
    dumpln("  == getChildValue: found no "+tagName+" children of node "+node.tagName);
  }

  return retVal;
}

/***
Given: 
  <a>
    <c foo='bar'>111</c>
    <c foo='bally'>222</c>
    <c foo='bax'>333</c>
  </a>
  
Calling getChildValueWhereAttrEquals(a,"foo","bally");
Will return 222;
    
***/
function getChildValueWhereAttrEquals(parentNode,attrName,attrValue)
{
  var gclctr=0;
  var cur=null;

  for(gclctr=0;gclctr<parentNode.childNodes.length;gclctr++){
    cur = parentNode.childNodes.item(gclctr);
    if(cur.nodeType == 1&& attrValue == cur.getAttribute(attrName)){
      if(cur.firstChild!=null){
        return cur.firstChild.nodeValue;
        //return getDomValue(cur); // return the desired value
      }
      else{
        return null;
      }
    }
  }
  return null;
}

/***
Given: 
  <a>
    <c foo='bar'>111</c>
    <c foo='bally'>222</c>
    <c foo='bax'>333</c>
  </a>
  
Calling getChildValueWhereAttrEquals(a,"foo","bally");
Will return the node containing 222;
    
***/
function getChildNodeWhereAttrEquals(parentNode,attrName,attrValue)
{
  var gclctr=0;
  var cur=null;

  for(gclctr=0;gclctr<parentNode.childNodes.length;gclctr++){
    cur = parentNode.childNodes.item(gclctr);
    if(cur.nodeType == 1&& attrValue == cur.getAttribute(attrName)){
      return cur;
    }
  }
  return null;
}



/***
sets the value of a node that contains text. where tag name is a tag inside of node whose
value should be set to newValue
***/
function setChildValue(node, tagName, newValue)
{
  var retVal = null;
  var root = node.ownerDocument;

  try{
    if(node==null){
      dumpln("setChildValue: bad node "+node);
      getStackTrace();
      return null;
    }
    nl = getChildList(node,tagName);
    if(nl==null){
      dumpln("setChildValue: bad child list "+nl);
      getStackTrace();
      return null;
    }
    var workingTag = nl[0];

    if(workingTag==null){
      //--- put in new tag to hold the value
      workingTag = root.createElementNS(null,tagName);
      node.appendChild(workingTag);
    }
    else{
      //--- remove the old value
      while(workingTag.childNodes != null && workingTag.childNodes.length>0){
        workingTag.removeChild(workingTag.childNodes.item(0));
      }
    }
    //--- set the value in place
    workingTag.appendChild(root.createTextNode(newValue));

  }
  catch(e){
    dumpln("setChildValue: "+e);
    dumpln("input: "+node+","+tagName+","+newValue);
    getStackTrace();
  }
}



/***
skips some useless dom garbage and gets the text node value out of a node that contains text.
Pass the text node into other objects and use it as your model to ensure your changes are
applied to the tree
***/
function getChildNode(node, tagName)
{
  var retVal = null;

  try{
    if(node==null){
      dumpln("getChildValue: bad node "+node);
      getStackTrace();
      return null;
    }
    nl = getChildList(node,tagName);
    if(nl==null){
      dumpln("getChildValue: bad child list "+nl);
      getStackTrace();
      return null;
    }
    if(nl[0]==null){
      return null;
    }
  
    retVal = nl[0];
  }
  catch(e){
    dumpln("getChildValue: "+e);
    dumpln("input: "+node+","+tagName);
    getStackTrace();
  }

  /*
  if(retVal==null){
    dumpln("getChildValue: found no "+tagName+" children of node "+node.tagName);
    getStackTrace();
  }
  */

  return retVal;
}




/***
doms getChildrenByTagName method if stupid and it's rotten.  I don't care about my
childrens, childrens, childrens, children!!!
***/
function getChildList(parentNode, childTagName)
{
  var retList = new Array();

  try{
    if(parentNode==null){
      dumpln("getChildList: null parent node");
      getStackTrace();
      return null;
    }

    for(gclctr=0;gclctr<parentNode.childNodes.length;gclctr++){
      if(parentNode.childNodes.item(gclctr).nodeType == 1 && (
           parentNode.childNodes.item(gclctr).tagName == childTagName ||
           childTagName == null
         )
        ){
        retList[retList.length] = parentNode.childNodes.item(gclctr);
      }
    }
  }
  catch(e){
    dumpln("getChildList: "+e);
    dumpln("input: "+parentNode+","+childTagName);
    getStackTrace();
  }

  return retList;
}


/***
of the two choices, returs the one that doesn't match curValue.
***/
function toggle(curValue, choiceA, choiceB)
{
  if(curValue==choiceA){
    return choiceB;
  }
  else{
    return choiceA;
  }
}


/***
The ability for javascript to do a stack trace is a gift from god to the 
lowly men and women of this earth.  I am humbly ecstatic.
***/
function getStackTrace()
{
  dumpln("=-=-=-=-=-=-=-=-=-=-=-=-=-=-");

  var frame = Components.stack.caller;
  while (frame){
    dumpln( "frame:"+frame );
    frame = frame.caller;
  }
  dumpln("=-=-=-=-=-=-=-=-=-=-=-=-=-=-");
}

/***
given an id of the form 'sometext:id' return 'parm1:id'

This is useful for giving type to id numbers used through the system.
If I have 20 widgets all relating to a form question with id 10 I would like to
give them all id="10".  However getById only allows one winner, so they have to be
disambiguated.  Therefore they all get type:id.  Thus the form-question object might get
fq:10 where the move button on the form question object might get mb:10.

This method allows you to give an id of type mb:10 and get fq:10.  If you just want the
10 pass in null for the second parameter.

If you pass   'fq:10','mq:'     you'll get 'mq:10'
If you pass   '10','mq:'        you'll get 'mq:10'
If you pass   'fq:10',null      you'll get '10'

remember that newPrefix MUST contain the colon.
***/
function alterId(input, newPrefix)
{
  try{
    if(input==null){
      dumpln("alterId: input was null");
      getStackTrace();
    }
  
    if(newPrefix==null){
      newPrefix="";
    }

    input = ""+input; // ensure the input is a string
  
    questionIdList = input.split(":");
    if(questionIdList.length>1){
      input = questionIdList[1];
      return newPrefix+questionIdList[1];
    }
    else{
      return newPrefix+questionIdList[0];
    }
  }
  catch(e){
    dumpln("alterId: "+e);
    getStackTrace();
  }
}




/***
Remove the children of a given tag name from the given node.
***/
function clearChildren(nodeToEmpty, targetTagName)
{
  try{
    var nukeList = getChildList(nodeToEmpty, targetTagName);
    var lctr = 0;
    for(lctr=0;lctr<nukeList.length;lctr++){
      nukeList[lctr].parentNode.removeChild(nukeList[lctr]);
    }
  }
  catch(ex){
    dumpln("clearChildren: "+ex);
    getStackTrace();
  }
}

/****
Saves a few lines of code by creating, setting an attribute and appending in one call.
****/
function appendTagAndSet(parentNode, tagName, attribName, attribVal)
{
  try{
    var localNode = parentNode.ownerDocument.createElement(tagName);
    if(attribName!=null){
      localNode.setAttribute(attribName,attribVal);
    }
    parentNode.appendChild(localNode);
    return localNode;
  }
  catch(ex){
    dumpln("prest");
    getStackTrace();
    dumpln("post st");
    dumpln("appendTagAndSet "+ex);
  }
  return null;
}

/****
Saves a few lines of code by creating, setting the value and appending in one call.
****/
function appendTagWithValue(parentNode, tagName, tagValue)
{
  var localNode = parentNode.ownerDocument.createElement(tagName);
  localNode.appendChild(localNode.ownerDocument.createTextNode(tagValue));
  parentNode.appendChild(localNode);
  return localNode;
}


//====== Link definition utilities.

/***
Gets the far side out of a link definition. 
Note that the near side is assumed to be curObjectName
***/
function getFarSideForLinkDef(linkDef,nearSide)
{
  //--- put a list into linkObjectMap to list the far objects for this link code.
  //find the far object type
  var childElementList = getChildList(linkDef,null);
  if(nearSide == null){
    nearSide = curObjectName;
  }

  var ectr;
  var farObjectType;
  for(ectr=0;ectr<childElementList.length;ectr++){
    if( -1 != childElementList[ectr].tagName.indexOf("-side") &&
        -1 == childElementList[ectr].tagName.indexOf(nearSide) ){
      //--- it's a side, but not the current object side.
      return firstChild(childElementList[ectr]).tagName;
    }
  }
  return null;
}

/***
Gets the far side out of a link definition. 
Note that the near side is assumed to be curObjectName
***/
function getFirstLinkSideNode(linkDef)
{
  //--- put a list into linkObjectMap to list the far objects for this link code.
  //find the far object type
  var childElementList = getChildList(linkDef,null);
  var ectr;
  var farObjectType;
  for(ectr=0;ectr<childElementList.length;ectr++){
    if( -1 != childElementList[ectr].tagName.indexOf("-side") ){
      //--- it's a side, but not the current object side.
      var sideNode = childElementList[ectr];
      var tagName = sideNode.tagName.slice(0, sideNode.tagName.length-5);
      dumpln("using tagName "+tagName);
      return getChildNode(sideNode,tagName);
    }
  }
  return null;
}


/***
helper for sortLinkDefListForDisplay
***/
function sortLinkDefListForDisplayComparitor(a,b)
{
  var aText = getFullLinkDefName(getDANode(a));
  var bText = getFullLinkDefName(getDANode(b));
  return aText.localeCompare(bText);
}


/***
Puts the list in the appropriate order for display.  
(sorting on LinkCode:FarObjectType)
***/
function sortLinkDefListForDisplay(itemList)
{
  itemList.sort(sortLinkDefListForDisplayComparitor);
}

/***
Puts the list in the appropriate order for display.  
(sorting on LinkCode:FarObjectType)
***/
function linkDefListContains(itemList, testLinkDefName)
{
  var ctr =0;
  for(ctr=0;ctr<itemList.length;ctr++){
    var key = itemList[ctr];
    var curText = getFullLinkDefName(getDANode(key));
    if(curText==testLinkDefName){
      return true;
    }
  }
  return false;
}


/***
Gets the string name for a link definition
***/
function getFullLinkDefName(linkDef)
{
  var linkCode = getChildValue(linkDef,"link-code");
  var linkObj  = getFarSideForLinkDef(linkDef,"link-code");
  return linkCode+":"+linkObj;
}

/***
Puts the list in the appropriate order for display.  
(sorting on DA Node ID which tells us what order they were found/created in.
 this is important if a link is undefined then re-defined)
***/
function sortLinkDefListForPublish(itemList)
{
  itemList.sort();
}


//=========================================== Text editing help =====

/***
This is a global variable that lets me know what label is currently being edited.
Only one label can be edited at a time.
***/
var curEditParent = null;


/***
Starting the edit process is like taking focus from a previous edit label and moving it to a new one.
The only one is no longer editable (and thus changes back into a label from a textbox.
The newly clicked label becomes a textbox.  When in a textbox if the user presses enter the 
changes are accepted and submitted to DOM.  If the users presses escapge the changes are abandoned.
***/
function startEdit(parentNode)
{
  if(curEditParent!=null){
    //--- if the user clicks somewhere else, accept the edit
    stopEdit(true);
  }

  var labelTag = firstChild(parentNode);
  var editTag  = appendTagAndSet(parentNode, "textbox", "id", "inEdit")
  editTag.setAttribute("value", labelTag.getAttribute("value"));
  editTag.setAttribute("size", "60");
  editTag.setAttribute("onkeypress",
                       "if(event.keyCode==13){stopEdit(true);} if(event.keyCode==27){stopEdit(false);}");
  //editTag.setAttribute("focused", "true");
  editTag.select();
  parentNode.removeChild(labelTag);
  curEditParent = parentNode;
}

/***
When an edit label should no longer be edited, this method is called.  It will either accept or abondon
the changes made to the tag.  Note that this method MUST be called when leaving the context of an object
and before publishing an object.  If calling because of a context change you must send false as the 
parameter or you risk having the wrong object get marked as edited
***/
function stopEdit(acceptFlag)
{
  if(curEditParent == null){
    return;
  }
  var editTag = firstChild(curEditParent);

  if(acceptFlag){
    //--- set the value into dom
    setDANodeValue(curEditParent.getAttribute("daNodeId"),editTag.value);
    declareCurObjectHasChanged();
  }

  //--- get the value out of dom and make a label out of it.
  curEditParent.removeChild(editTag);
  var labelTag  = appendTagAndSet(curEditParent, "label", "value", getDANodeValue(curEditParent.getAttribute("daNodeId")));
  labelTag.setAttribute("onclick","startEdit(this.parentNode);");
  labelTag.setAttribute("onfocus","this.setAttribute('class','focus-friendly-on');");
  labelTag.setAttribute("onblur", "this.setAttribute('class','focus-friendly');" );
  labelTag.setAttribute("class",  "focus-friendly");
  labelTag.setAttribute("onkeypress","if(event.charCode==32 || event.keyCode==13){startEdit(this.parentNode);}");
  labelTag.focus();

  curEditParent = null;
}



//============================= END special tag help ====================
